package data.campaign.dracomanager;

import com.fs.starfarer.api.EveryFrameScript;
import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import com.fs.starfarer.api.impl.campaign.intel.BaseEventManager;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import data.campaign.dracomanager.Draco_RaiderBaseIntel.RaiderBaseTier;

import java.util.Random;

import static data.campaign.dracomanager.Draco_RaiderAndBaseConfig.*;

public class Draco_RaiderBaseManager extends BaseEventManager {

    public static final String KEY = "$" + DRACO_FACTION_ID + "_RaiderBaseManager";

    public static Draco_RaiderBaseManager getInstance() {
        Object test1 = Global.getSector().getMemoryWithoutUpdate().get(KEY);
        return (Draco_RaiderBaseManager) test1;
    }

    protected long start = 0;
    protected float extraDays = 0;

    protected int numDestroyed = 0;

    public Draco_RaiderBaseManager() {
        super();
        Global.getSector().getMemoryWithoutUpdate().set(KEY, this);
        start = Global.getSector().getClock().getTimestamp();
    }

    @Override
    protected int getMinConcurrent() {
        return MIN_DRACO_CAMP;
    }

    @Override
    protected int getMaxConcurrent() {
        return MAX_DRACO_CAMP;
    }

    @Override
    protected float getBaseInterval() {
        return DRACO_CAMP_CHECK_DAYS;
    }

    protected Random randomForRaiderBaseEvent = new Random();

    @Override
    protected EveryFrameScript createEvent() {

        if (this.isDone()) {
            return null;
        }

        if (randomForRaiderBaseEvent.nextFloat() < DRACO_CAMP_CHECK_PROB) {
            return null;
        }

        StarSystemAPI system = pickSystemForRaiderBase();
        if (system == null) {
            return null;
        }

        RaiderBaseTier tier = pickRaiderBaseTier();

        String factionId = DRACO_FACTION_ID;
        if (factionId == null) {
            return null;
        }

        Draco_RaiderBaseIntel intel = new Draco_RaiderBaseIntel(system, factionId, tier);
        if (intel.isDone()) {
            intel = null;
        }

        return intel;
    }

    public float getDaysSinceStart() {
        float days = Global.getSector().getClock().getElapsedDaysSince(start) + extraDays;
        if (Misc.isFastStartExplorer()) {
            days += 180f - 30f;
        } else if (Misc.isFastStart()) {
            days += 180f + 60f;
        }
        return days;
    }

    public float getExtraDays() {
        return extraDays;
    }

    public void setExtraDays(float extraDays) {
        this.extraDays = extraDays;
    }

    protected RaiderBaseTier pickRaiderBaseTier() {
        float days = getDaysSinceStart();

        days += numDestroyed * 200;

        WeightedRandomPicker<RaiderBaseTier> picker = new WeightedRandomPicker<>();

        if (days < 360) {
            picker.add(RaiderBaseTier.TIER_1_1MODULE, 10f);
            picker.add(RaiderBaseTier.TIER_2_1MODULE, 10f);
        } else if (days < 720f) {
            picker.add(RaiderBaseTier.TIER_2_1MODULE, 10f);
            picker.add(RaiderBaseTier.TIER_3_2MODULE, 10f);
        } else if (days < 1080f) {
            picker.add(RaiderBaseTier.TIER_3_2MODULE, 10f);
            picker.add(RaiderBaseTier.TIER_4_3MODULE, 10f);
        } else {
            picker.add(RaiderBaseTier.TIER_3_2MODULE, 10f);
            picker.add(RaiderBaseTier.TIER_4_3MODULE, 10f);
            picker.add(RaiderBaseTier.TIER_5_3MODULE, 10f);
        }

        return picker.pick();
    }

    protected StarSystemAPI pickSystemForRaiderBase() {
        WeightedRandomPicker<StarSystemAPI> far = new WeightedRandomPicker<>(randomForRaiderBaseEvent);
        WeightedRandomPicker<StarSystemAPI> picker = new WeightedRandomPicker<>(randomForRaiderBaseEvent);

        for (StarSystemAPI system : Global.getSector().getStarSystems()) {
            float days = Global.getSector().getClock().getElapsedDaysSince(system.getLastPlayerVisitTimestamp());
            if (days < 45f) {
                continue;
            }

            float weight = 0f;
            if (system.hasTag(Tags.THEME_MISC_SKIP)) {
                weight = DRACO_BASE_LOC_MISCSKIP;
            } else if (system.hasTag(Tags.THEME_MISC)) {
                weight = DRACO_BASE_LOC_MISC;
            } else if (system.hasTag(Tags.THEME_REMNANT_NO_FLEETS)) {
                weight = DRACO_BASE_LOC_REMNANTNOFLEETS;
            } else if (system.hasTag(Tags.THEME_RUINS)) {
                weight = DRACO_BASE_LOC_RUINS;
            } else if (system.hasTag(Tags.THEME_CORE_UNPOPULATED)) {
                weight = DRACO_BASE_LOC_COREUNPOPULATED;
            }
            if (weight <= 0f) {
                continue;
            }

            float usefulStuff = system.getCustomEntitiesWithTag(Tags.OBJECTIVE).size()
                    + system.getCustomEntitiesWithTag(Tags.STABLE_LOCATION).size();
            if (usefulStuff <= 0) {
                continue;
            }

            if (Misc.hasPulsar(system)) {
                continue;
            }
            if (Misc.getMarketsInLocation(system).size() > 0) {
                continue;
            }

            float dist = system.getLocation().length();

            float distMult = 1f;

            if (dist > 36000f) {
                far.add(system, weight * usefulStuff * distMult);
            } else {
                picker.add(system, weight * usefulStuff * distMult);
            }
        }

        if (picker.isEmpty()) {
            picker.addAll(far);
        }

        return picker.pick();
    }

    public int getNumDestroyed() {
        return numDestroyed;
    }

    public void setNumDestroyed(int numDestroyed) {
        this.numDestroyed = numDestroyed;
    }

    public void incrDestroyed() {
        numDestroyed++;
    }

}
