package data.campaign.fangmanager;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.StarSystemAPI;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.campaign.econ.MarketConditionAPI;
import com.fs.starfarer.api.campaign.econ.MarketConditionPlugin;
import com.fs.starfarer.api.impl.campaign.DebugFlags;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Tags;
import com.fs.starfarer.api.impl.campaign.intel.BaseIntelPlugin;
import com.fs.starfarer.api.ui.Alignment;
import com.fs.starfarer.api.ui.SectorMapAPI;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.Misc;

import java.awt.*;
import java.util.Set;

import static data.campaign.fangmanager.Fang_RaiderAndBaseConfig.*;
import static data.campaign.fangmanager.Fang_RaiderBaseIntel.log;

public class Fang_RaiderActivityConditionIntel extends BaseIntelPlugin {

	protected StarSystemAPI system;
	protected Fang_RaiderBaseIntel source;
	
	public Fang_RaiderActivityConditionIntel(StarSystemAPI system, Fang_RaiderBaseIntel source) {
		this.system = system;
		this.source = source;
		
		
		boolean hasPlayerMarkets = false;
		for (MarketAPI curr : Global.getSector().getEconomy().getMarkets(system)) {
			hasPlayerMarkets |= curr.isPlayerOwned();
		}
		if (!hasPlayerMarkets) {
			setPostingLocation(system.getCenter());
		}
		
		Global.getSector().addScript(this);
		
		if (!Misc.getMarketsInLocation(system, Factions.PLAYER).isEmpty()) {
			Global.getSector().getIntelManager().addIntel(this);
		} else {
			Global.getSector().getIntelManager().queueIntel(this);
		}
	}
	
	@Override
	public boolean canMakeVisibleToPlayer(boolean playerInRelayRange) {
		if (DebugFlags.SEND_UPDATES_WHEN_NO_COMM && source.isPlayerVisible()) return true;
		return super.canMakeVisibleToPlayer(playerInRelayRange);
	}


	public Fang_RaiderBaseIntel getSource() {
		return source;
	}

	@Override
	protected void notifyEnded() {
		super.notifyEnded();
		Global.getSector().removeScript(this);
	}


	@Override
	protected void notifyEnding() {
		super.notifyEnding();
		
		for (MarketAPI curr : source.getAffectedMarkets(system)) {
			if (curr.hasCondition(FANG_ACTIVITY_CONDITION_ID)) {
				curr.removeCondition(FANG_ACTIVITY_CONDITION_ID);
                                log.info(String.format("Removing raider activity condition in [%s], market: %s", system.getName(), curr.getName()));
			}
		}
	}

	@Override
	protected void advanceImpl(float amount) {
		super.advanceImpl(amount);
		
		if (source.isEnding() || source.getTarget() != system) {
			endAfterDelay();
			if (DebugFlags.SEND_UPDATES_WHEN_NO_COMM || Global.getSector().getIntelManager().isPlayerInRangeOfCommRelay()) {
				sendUpdateIfPlayerHasIntel(new Object(), false);
			}
			return;
		}
		
		for (MarketAPI curr : source.getAffectedMarkets(system)) {
			if (!curr.hasCondition(FANG_ACTIVITY_CONDITION_ID)) {
				curr.addCondition(FANG_ACTIVITY_CONDITION_ID, source);
                                log.info(String.format("Adding raider activity condition in [%s], market: %s", system.getName(), curr.getName()));
			}
		}
	}

	protected void addBulletPoints(TooltipMakerAPI info, ListInfoMode mode) {
		
		Color h = Misc.getHighlightColor();
		Color g = Misc.getGrayColor();
		float pad = 3f;
		float opad = 10f;
		
		float initPad = pad;
		if (mode == ListInfoMode.IN_DESC) initPad = opad;
		
		Color tc = getBulletColorForMode(mode);
		
		bullet(info);
		boolean isUpdate = getListInfoParam() != null;
		
		unindent(info);
	}
	
	@Override
	public void createIntelInfo(TooltipMakerAPI info, ListInfoMode mode) {
		Color c = getTitleColor(mode);
		info.addPara(getName(), c, 0f);
		addBulletPoints(info, mode);
	}
	
	@Override
	public void createSmallDescription(TooltipMakerAPI info, float width, float height) {
		Color h = Misc.getHighlightColor();
		Color g = Misc.getGrayColor();
		Color tc = Misc.getTextColor();
		float pad = 3f;
		float opad = 10f;
		
		info.addImage(getFactionForUIColors().getLogo(), width, 128, opad);
		
		info.addPara(FANG_FACTION.getDisplayName() + " raiders have been targeting colonies and shipping " +
				"in the " + system.getNameWithLowercaseType() + ".", opad);
		
		if (source.isPlayerVisible()) {
			info.addPara(FANG_FACTION.getDisplayNameWithArticle() + " raiders are based out of " +
					source.getMarket().getName() + " in the " + source.getSystem().getNameWithLowercaseType() + ".", opad);
		} else {
			float distLY = Misc.getDistanceLY(system.getLocation(), source.getSystem().getLocation());
			if (distLY < 10) {
				info.addPara("The location of the " + FANG_FACTION.getDisplayName() + " camp is unknown, but it's likely to be somewhere nearby.", opad);
			} else {
				info.addPara("The location of the " + FANG_FACTION.getDisplayName() + " camp is unknown, but there are indications that it's quite distant.", opad);
			}
		}
		
		info.addSectionHeading("Colonies affected", getFactionForUIColors().getBaseUIColor(),
							  getFactionForUIColors().getDarkUIColor(), Alignment.MID, opad);
		
		MarketConditionAPI condition = null;
		float initPad = opad;
		for (MarketAPI curr : source.getAffectedMarkets(system)) {
			if (condition == null) {
				condition = curr.getCondition(FANG_ACTIVITY_CONDITION_ID);
			}
			
			addMarketToList(info, curr, initPad);
			initPad = 0f;
		}
		
		if (condition != null) {
			MarketConditionPlugin plugin = condition.getPlugin();
			
			((Fang_RaiderActivityCondition)plugin).createTooltipAfterDescription(info, true);
		}
		
	}
	
	
	public StarSystemAPI getSystem() {
		return system;
	}

	@Override
	public String getIcon() {
		return FANG_ACTIVITY_INTEL_ICON;
	}
	
	@Override
	public Set<String> getIntelTags(SectorMapAPI map) {
		Set<String> tags = super.getIntelTags(map);
		tags.add(FANG_FACTION_ID);
		
		if (!Misc.getMarketsInLocation(system, Factions.PLAYER).isEmpty()) {
			tags.add(Tags.INTEL_COLONIES);
		}
		
		return tags;
	}
	
	public String getSortString() {
		String base = Misc.ucFirst(getFactionForUIColors().getPersonNamePrefix());
		return base + " C";
	}
	
	public String getName() {
		String base = FANG_ACTIVITY_INTEL_NAME;
		if (isEnding()) {
			return base + " - Over";
		}
		return base + " - " + system.getBaseName();
	}
	
	@Override
	public FactionAPI getFactionForUIColors() {
		return source.getFactionForUIColors();
	}

	public String getSmallDescriptionTitle() {
		return getName();
	}

	@Override
	public SectorEntityToken getMapLocation(SectorMapAPI map) {
		return system.getCenter();
	}

	
	@Override
	public String getCommMessageSound() {
		return getSoundMinorMessage();
	}
	
	
}







