package data.campaign.fangmanager;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.FactionAPI;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType;
import com.fs.starfarer.api.impl.campaign.ids.Conditions;
import com.fs.starfarer.api.impl.campaign.ids.Factions;
import com.fs.starfarer.api.impl.campaign.ids.Industries;
import java.util.ArrayList;
import java.util.Arrays;

/**
 * @author Vayra <<-Appropos to this glorious man
 */
public class Fang_RaiderAndBaseConfig {
    
    // set this to false unless you like opening campaigns to 100,000 raids
    public static final boolean FANG_RAID_DEBUG = false;
    
    /////////////////////////////////////
    // RAID INTEL/ACTION STRING CONFIG //
    /////////////////////////////////////
    
    // raider activity intel event display name:
    public static final String FANG_ACTIVITY_INTEL_NAME = "Fang Hunt";
    
    // raider activity market condition id and display name:
    // make sure this is set up in market_conditions.csv and points to the right script
    // look at the pirate activity line for an example
    public static final String FANG_ACTIVITY_CONDITION_ID = "hmi_fang_raid_activity";
    public static final String FANG_ACTIVITY_CONDITION_NAME = "Fang Hunt";    
    
    // filepaths goes in settings.json under the graphics > intel bit:
    public static final String FANG_ACTIVITY_INTEL_ICON = Global.getSettings().getSpriteName("intel", "fang_raid");
    public static final String FANG_BASE_INTEL_ICON = Global.getSettings().getSpriteName("intel", "fang_base");
    
    // Strings that appear in nonevent raider fleet mouseover, or intel panel:
    public static final String FANG_ACTION_TEXT = "hunting prey from"; // i.e. "raiding"
    public static final String FANG_STANDDOWN_TEXT1 = "rejoicing in a successful hunt"; // 69% chance, i.e. "standing down"
    public static final String FANG_STANDDOWN_TEXT2 = "celebrating a wildly successful hunt"; // 31% chance, i.e. "distributing ill-gotten goods"
    
    // list of base name parts like "Habitat" or "Hideaway", will get a random proper first name added to it
    // not sure where the first names are pulled from
    public static final ArrayList<String> FANG_BASE_TYPE_NAMES = new ArrayList<>(Arrays.asList(
            "Armory",
            "Barrow",
			"Butchery",
			"Camp",
			"Castle",
            "Catacomb",
            "Charnel",
            "Citadel",
            "Congregation",
			"Collective",
            "Crypt",
            "Demesne",
			"Delicatessen",
			"Eatery",
			"Eyrie",
			"Farm",
            "Flock",
            "Fortress",
            "Funerary",
			"Galley",
            "Grave",
            "Hall",
            "Haven",
			"House",
			"Kitchen",
            "Memorial",
            "Monument",
            "Necropolis",
			"Nest",
            "Ossuarium",
            "Ossuary",
			"Paddock",
			"Park",
			"Plain",
			"Pod",
            "Refuge",
            "Remembrance",
			"Reserve",
			"Restaurant",
            "Retreat",
            "Safehold",
            "Sanctuary",
            "Shrine",
            "Star Fortress",
            "Starhold",
            "Temple",
            "Tomb",
            "Vestry",
            "Ward"));
    
    // set your names for the "merc" fleet types in your .faction file 
    // look at Independents for an example
    // size order:
    // scout
    // bounty hunter
    // privateer
    // patrol
    // armada
    // doing it this way allows you to spawn raid fleets with different names than patrol fleets
    
    // i didn't want to do all the pronoun fuckery here so make sure you also edit the bar event text in the RaiderBaseRumorBarEvent!
    
    ////////////////////////
    // RAIDER CAMP CONFIG //
    ////////////////////////
    
    // min and max raid camps at a time:

    public static final int MIN_FANG_CAMP = 0;
    public static final int MAX_FANG_CAMP = 1; 
    
    // days between checks, and probability of check (for if we need to spawn more raider bases):
    public static final float FANG_CAMP_CHECK_DAYS = 90f;
    public static final float FANG_CAMP_CHECK_PROB = 0.05f;
    
    // pick weights for themes of systems to put a base in:
    // these are per-system, not per-theme
    // i.e. "core unpopulated" isn't disporportionately weighted don't worry
    // no i don't know what the themes are or mean
    public static final float FANG_BASE_LOC_MISCSKIP = 1f;
    public static final float FANG_BASE_LOC_MISC = 3f;
    public static final float FANG_BASE_LOC_REMNANTNOFLEETS = 3f;
    public static final float FANG_BASE_LOC_RUINS = 6f;
    public static final float FANG_BASE_LOC_COREUNPOPULATED = 3f;
    
    // initial market size of raider bases:
    // default for pirate and pathers is 3 (thousands of people)
    public static final int FANG_BASE_SIZE = 3;
    
    // months before a base upgrades if left alone:
    public static final float FANG_BASE_UPGRADE_MONTHS = 10f;
    
    // orbital station tech level spawn weight for raider bases
    // overridden if you have "pirateBaseStationTypes" set in the .faction file
	
	public static final float FANG_BASE_LOWTECH_WEIGHT = 5f;
	public static final float FANG_BASE_MIDLINE_WEIGHT = 1f;
	public static final float FANG_BASE_HIGHTECH_WEIGHT= 0f;
	
    // condition/industry ids to add to every raider market:
    // don't include population we handle that automagically from the BASE_SIZE above
    // same for orbital station, we autobuild one from the above
    public static final ArrayList<String> FANG_MARKET_CONDITIONS = new ArrayList<>(
            Arrays.asList(
                    Conditions.FRONTIER,
                    Conditions.NO_ATMOSPHERE,
                    Industries.POPULATION,
                    Industries.SPACEPORT,
                    Industries.MILITARYBASE,
                    Industries.ORBITALWORKS));
        
    /////////////////
    // RAID CONFIG //
    /////////////////
    
    // raider faction id:
    public static final String FANG_FACTION_ID = "fang";
    
    // faction we hate the most id:
    public static final String RAID_PRIMARY_TARGET_ID = Factions.HEGEMONY; 
    // (leave this as is and tweak the next couple settings if you want to raid all hostiles more or less equally)
    // (also if the player is hostile they're included as another primary target for event raids)
    // (note that event raids are also affected by settings.json noPirateRaidDays)
    // (so they wont wreck the player's colonies - or appear at all - for the first ~year by default)
    
    // raid everyone or just the person we hate the most?:
    // (non-event raids spawns silently from any of our markets up to our total normal market size, even from day one)
    // (event raids are bigger, spawn from our secret bases once noPirateRaidDays is past, and create the intel for an incoming raid)
    public static final boolean NON_EVENT_RAID_EVERYONE = false; 
    public static final boolean EVENT_RAID_EVERYONE = true;
    
    // multiplier to target weight when it's the faction we hate the most (or the player, if we're hostile):
    public static final Float RAID_PRIMARY_TARGET_WEIGHT = 1f;     // set to 1f if you want to raid all hostiles more or less equally
        
    // minimum Fleet Points for non-event raids, in order of size -- max = this * 2
    public static final Float FP_SCOUT = 15f; 
    public static final Float FP_BOUNTY_HUNTER = 30f; 
    public static final Float FP_PRIVATEER = 60f; 
    public static final Float FP_PATROL = 90f; 
    public static final Float FP_ARMADA = 180f; 
    
    // spawn weights for non-event raids, in order of size 
    public static final Float SPAWN_WEIGHT_SCOUT = 1f; 
    public static final Float SPAWN_WEIGHT_BOUNTY_HUNTER = 1f; 
    public static final Float SPAWN_WEIGHT_PRIVATEER = 3f; 
    public static final Float SPAWN_WEIGHT_PATROL = 5f; 
    public static final Float SPAWN_WEIGHT_ARMADA = 1f; 

    // average Fleet Points for event raids, in order of size 
    // will be modified +/- 0-25%, then +/- 0-25% again ... wtf alex lol
    public static final Float RAID_TIER1_FP = 100f; 
    public static final Float RAID_TIER2_FP = 175f; 
    public static final Float RAID_TIER3_FP = 250f; 
    public static final Float RAID_TIER4_FP = 500f; 
    public static final Float RAID_TIER5_FP = 800f; 
    // these are the default values for pirates! wow!
    
    ////////////////////////////////////////////////////////////////////////////
    /////                  stuff you don't need to change                  /////
    ////////////////////////////////////////////////////////////////////////////
    
    public static final MercType SCOUT = MercType.SCOUT;
    public static final MercType BOUNTY_HUNTER = MercType.BOUNTY_HUNTER;
    public static final MercType PRIVATEER = MercType.PRIVATEER;
    public static final MercType PATROL = MercType.PATROL;
    public static final MercType ARMADA = MercType.ARMADA;
    public static final FactionAPI FANG_FACTION = Global.getSector().getFaction("fang");
    
}

