package data.campaign.fangmanager;

import org.lwjgl.util.vector.Vector2f;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FleetAssignment;
import com.fs.starfarer.api.campaign.LocationAPI;
import com.fs.starfarer.api.campaign.SectorEntityToken;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.fleets.BaseLimitedFleetManager;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.MercType;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import java.util.ArrayList;
import java.util.List;

import static data.campaign.fangmanager.Fang_RaiderAndBaseConfig.*;

public class Fang_RaiderNonEventFleetManager extends BaseLimitedFleetManager {

    	public static class NonEventRaiderManagedFleetData {
		public float startingFleetPoints = 0;
		public CampaignFleetAPI fleet;
		public NonEventRaiderManagedFleetData(CampaignFleetAPI fleet) {
			this.fleet = fleet;
		}
	}
	
protected List<NonEventRaiderManagedFleetData> fleetsActive = new ArrayList<>();

    @Override
    protected int getMaxFleets() {
        int count = 0;
        for (MarketAPI market : Global.getSector().getEconomy().getMarketsCopy()) {
            String fid = market.getFactionId();
            if (fid.equals(FANG_FACTION_ID)) {
                count += market.getSize();
            }
        }
        return count;
    }

    public CampaignFleetAPI spawnFleet(MarketAPI spawnMarket, MercType type, MarketAPI target) {
        if (target == null) {
            target = pickTarget();
            if (target == null) {
                return null;
            }
        }
        if (spawnMarket == null) {
            WeightedRandomPicker<MarketAPI> picker2 = new WeightedRandomPicker<>();
            for (MarketAPI market2 : Global.getSector().getEconomy().getMarketsCopy()) {
                String fid = market2.getFactionId();
                if (fid.equals(FANG_FACTION_ID)) {
                    picker2.add(market2, market2.getSize());
                }
            }

            spawnMarket = picker2.pick();
            if (spawnMarket == null) {
                return null;
            }
        }
        if (type == null) {
            WeightedRandomPicker<MercType> picker = new WeightedRandomPicker<>();
            picker.add(SCOUT, SPAWN_WEIGHT_SCOUT);
            picker.add(BOUNTY_HUNTER, SPAWN_WEIGHT_BOUNTY_HUNTER);
            picker.add(PRIVATEER, SPAWN_WEIGHT_PRIVATEER);
            picker.add(PATROL, SPAWN_WEIGHT_PATROL);
            picker.add(ARMADA, SPAWN_WEIGHT_ARMADA);

            type = picker.pick();
        }

        float combat = 0f;
        float tanker = 0f;
        float freighter = 0f;
        String fleetType = type.fleetType;
        switch (type) {
            case SCOUT:
                combat = Math.round(FP_SCOUT + (float) Math.random() * FP_SCOUT);
                break;
            case PRIVATEER:
                combat = Math.round(FP_PRIVATEER + (float) Math.random() * FP_PRIVATEER);
                break;
            case BOUNTY_HUNTER:
                combat = Math.round(FP_BOUNTY_HUNTER + (float) Math.random() * FP_BOUNTY_HUNTER);
                break;
            case PATROL:
                combat = Math.round(FP_PATROL + (float) Math.random() * FP_PATROL);
                break;
            case ARMADA:
                combat = Math.round(FP_ARMADA + (float) Math.random() * FP_ARMADA);
                break;
        }

        FleetParamsV3 params = new FleetParamsV3(
                spawnMarket,
                target.getLocation(), FANG_FACTION_ID, // quality will always be reduced by non-market-faction penalty, which is what we want 
                null,
                fleetType,
                combat, // combatPts
                freighter, // freighterPts 
                tanker, // tankerPts
                0f, // transportPts
                0f, // linerPts
                0f, // utilityPts
                0f // qualityMod
        );

        CampaignFleetAPI fleet = FleetFactoryV3.createFleet(params);
        log.info(String.format("Spawning fleet via SpawnFleet, spawnMarket: [%s], target: [%s], combat points: [%s]", spawnMarket.getName(), target.getName(), (int) combat));
        
        if (fleet == null || fleet.isEmpty()) {
        log.info(String.format("Fleet was null. Nevermind"));
            return null;
        }
        
        log.info(String.format("Spawned fleet via SpawnFleet, Actual Fleet Points: [%s], Fleet Size: [%s]", fleet.getFleetPoints(), fleet.getFleetSizeCount()));
        
        SectorEntityToken entity = spawnMarket.getPrimaryEntity();
        entity.getContainingLocation().addEntity(fleet);
        fleet.setLocation(entity.getLocation().x, entity.getLocation().y);
        MarketAPI source = Misc.getSourceMarket(fleet);
        if (source == null) {
        log.info(String.format("Source was null. Nevermind"));
            return null;
        }

        Vector2f dest = Misc.getPointAtRadius(target.getLocationInHyperspace(), 1500);
        LocationAPI hyper = Global.getSector().getHyperspace();
        SectorEntityToken token = hyper.createToken(dest.x, dest.y);

        fleet.addAssignment(FleetAssignment.GO_TO_LOCATION, token, 1000,
                "travelling to the " + target.getName() + " star system");
        fleet.addAssignment(FleetAssignment.ATTACK_LOCATION, target.getPrimaryEntity(), 20,
                FANG_ACTION_TEXT + target.getName());
        fleet.addAssignment(FleetAssignment.GO_TO_LOCATION, source.getPrimaryEntity(), 1000,
                "returning to " + source.getName());
        if ((float) Math.random() > 0.69f) {
            fleet.addAssignment(FleetAssignment.ORBIT_PASSIVE, source.getPrimaryEntity(), 2f + 2f * (float) Math.random(),
                    FANG_STANDDOWN_TEXT2);
        } else {
            fleet.addAssignment(FleetAssignment.ORBIT_PASSIVE, source.getPrimaryEntity(), 2f + 2f * (float) Math.random(),
                    FANG_STANDDOWN_TEXT1);
        }
        fleet.addAssignment(FleetAssignment.GO_TO_LOCATION_AND_DESPAWN, source.getPrimaryEntity(), 1000);

        return fleet;
    }

    protected MarketAPI pickTarget() {
        WeightedRandomPicker<MarketAPI> picker3 = new WeightedRandomPicker<>();
        for (MarketAPI market3 : Global.getSector().getEconomy().getMarketsCopy()) {
            String fid = market3.getFactionId();
            if (fid.equals(RAID_PRIMARY_TARGET_ID)) {
                picker3.add(market3, market3.getSize() * RAID_PRIMARY_TARGET_WEIGHT);
            } else if (NON_EVENT_RAID_EVERYONE) {
                picker3.add(market3, market3.getSize());
            }
        }
        return picker3.pick();
    }
    
@Override
    	public void advance(float amount) {
		float days = Global.getSector().getClock().convertToDays(amount);
		
		tracker.advance(days * getSpawnRateMult());
		if (!tracker.intervalElapsed()) return;
		
		float next = getNextInterval();
		tracker.setInterval(next, next);
		
		Global.getSettings().profilerBegin(this.getClass().getSimpleName() + ".advance()");
		List<NonEventRaiderManagedFleetData> remove = new ArrayList<>();
		for (NonEventRaiderManagedFleetData data : fleetsActive) {
			if (data.fleet.getContainingLocation() == null || !data.fleet.getContainingLocation().getFleets().contains(data.fleet)) {
				remove.add(data);
				log.info("Cleaning up orphaned fleet [" + data.fleet.getNameWithFaction() + "] of [" + data.fleet.getFleetSizeCount() + "] ships");
			}
		}
		fleetsActive.removeAll(remove);
		
		int max = getMaxFleets();
		
		if (fleetsActive.size() < max) {
			log.info(fleetsActive.size() + " out of a maximum " + max + " fleets in play for [" + getClass().getName() + "]");
			CampaignFleetAPI fleet = spawnFleet();
			if (fleet != null) {
				fleet.addEventListener(this);
				NonEventRaiderManagedFleetData data = new NonEventRaiderManagedFleetData(fleet);
				fleetsActive.add(data);
				log.info("Spawned nonevent raider fleet [" + fleet.getNameWithFaction() + "] at hyperloc " + fleet.getLocationInHyperspace());
			} else {
				log.info("Could not spawn fleet - returned null");
			}
		} else {
			log.debug("Maximum number of " + max + " fleets already in play for [" + getClass().getName() + "]");
		}
		Global.getSettings().profilerEnd();
	}
    
    @Override
    protected CampaignFleetAPI spawnFleet() {
        return spawnFleet(null, null, null);
    }
}
