package data.campaign.fleets;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.*;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.fleets.BaseLimitedFleetManager;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.impl.campaign.fleets.SourceBasedFleetManager;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import org.lazywizard.lazylib.MathUtils;

import java.util.Random;

public class DracoFleetManager extends SourceBasedFleetManager {

	protected int minPts;
	protected int maxPts;

	public DracoFleetManager(SectorEntityToken source, float thresholdLY, int minFleets, int maxFleets, float respawnDelay,
							int minPts, int maxPts) {
		super(source, thresholdLY, minFleets, maxFleets, respawnDelay);
		this.minPts = minPts;
		this.maxPts = maxPts;
	}

	@Override
	protected CampaignFleetAPI spawnFleet() {
		SectorEntityToken DracospawnEntity = Get_BadDragonSupplier();  // like a planet or such
		if (DracospawnEntity == null) {
			return null;
		}

		float combat = MathUtils.getRandomNumberInRange(1, MathUtils.getRandomNumberInRange(80,
				MathUtils.getRandomNumberInRange(
						100, 200)));

		String fleetType;
		if (combat < 10) {
			fleetType = FleetTypes.PATROL_SMALL;
		} else if (combat < 40) {
			fleetType = FleetTypes.PATROL_MEDIUM;
		} else {
			fleetType = FleetTypes.PATROL_LARGE;
		}

		MarketAPI dracomarket = Global.getFactory().createMarket(String.valueOf((new Random()).nextLong()),
				String.valueOf((new Random()).nextLong()), 4);

		FleetParamsV3 params = new FleetParamsV3(
				null, // market
				DracospawnEntity.getLocation(), // location
				"draco", // fleet's faction, if different from above, which is also used for source market picking
				1.5f,
				fleetType,
				combat, // combatPts
				0f, // freighterPts
				0f, // tankerPts
				0f, // transportPts
				0f, // linerPts
				0f, // utilityPts
				0 // qualityBonus
		);

		LocationAPI location = DracospawnEntity.getContainingLocation();  // a star system, or hyperspace
		CampaignFleetAPI fleet = FleetFactoryV3.createFleet(params);
		location.addEntity(fleet);// adds our new fleet into the containing location
		fleet.setLocation(DracospawnEntity.getLocation().x, DracospawnEntity.getLocation().y);// positions fleet on top of the spawn entity

		if ((fleet == null) || fleet.isEmpty()) {
			return null;
		}

		// setting the below means: transponder off and more "go dark" use when traveling
		//fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);

		fleet.addAssignment(FleetAssignment.ORBIT_AGGRESSIVE, DracospawnEntity, 3f + (float) Math.random() * 2f);
		if ((float) Math.random() > 0.75f) {
			fleet.addAssignment(FleetAssignment.RAID_SYSTEM, DracospawnEntity, 250,
					"raiding around the " + DracospawnEntity.getName() + " star system");
		} else {
			fleet.addAssignment(FleetAssignment.RAID_SYSTEM, DracospawnEntity, 500,
					"raiding the " + DracospawnEntity.getName() + " star system");
		}
		fleet.addAssignment(FleetAssignment.GO_TO_LOCATION, DracospawnEntity, 20,
				"returning to " + DracospawnEntity.getName());
		fleet.addAssignment(FleetAssignment.ORBIT_PASSIVE, DracospawnEntity, 2f + 2f * (float) Math.random(),
				"orbiting " + DracospawnEntity.getName());

		fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_FLEET_TYPE, "fangFleet");
		fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);
		fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_NO_MILITARY_RESPONSE, true);
		fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_FIGHT_TO_THE_LAST, true);

		return fleet;
	}


	protected SectorEntityToken Get_BadDragonSupplier() {
		WeightedRandomPicker<SectorEntityToken> picker = new WeightedRandomPicker<SectorEntityToken>();
		for (StarSystemAPI system : Global.getSector().getStarSystems()) {
			float weight = 0f;
			if (system.getName().equals("Obsidian")) continue;
			float w = 11f;
			if (w > weight) weight = w;
			for (SectorEntityToken entity : system.getAllEntities()) {
				if (entity.getId().equals("pyrite")) {
					picker.add(entity, weight);
				}
			}
		}
		return picker.pick();
	}
}
