package data.campaign.fleets;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.*;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.fleets.BaseLimitedFleetManager;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.impl.campaign.fleets.SourceBasedFleetManager;
import com.fs.starfarer.api.impl.campaign.ids.FleetTypes;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.WeightedRandomPicker;
import org.lazywizard.lazylib.MathUtils;

import java.util.Random;

public class FangFleetManager extends SourceBasedFleetManager {

	protected int minPts;
	protected int maxPts;

	public FangFleetManager(SectorEntityToken source, float thresholdLY, int minFleets, int maxFleets, float respawnDelay,
							int minPts, int maxPts) {
		super(source, thresholdLY, minFleets, maxFleets, respawnDelay);
		this.minPts = minPts;
		this.maxPts = maxPts;
	}

	@Override
	protected CampaignFleetAPI spawnFleet() {
		SectorEntityToken FangspawnEntity = Get_FursuitSupplier();  // like a planet or such
		if (FangspawnEntity == null) {
			return null;
		}

		Random random = new Random();

		float combat = MathUtils.getRandomNumberInRange(1, MathUtils.getRandomNumberInRange(40,
				MathUtils.getRandomNumberInRange(
						60, 100)));

		String fleetType;
		if (combat < 10) {
			fleetType = FleetTypes.PATROL_SMALL;
		} else if (combat < 40) {
			fleetType = FleetTypes.PATROL_MEDIUM;
		} else {
			fleetType = FleetTypes.PATROL_LARGE;
		}


		MarketAPI fangmarket = Global.getFactory().createMarket(String.valueOf((new Random()).nextLong()),
				String.valueOf((new Random()).nextLong()), 4);

		FleetParamsV3 params = new FleetParamsV3(
				null, // market
				FangspawnEntity.getLocation(), // location
				"fang", // fleet's faction, if different from above, which is also used for source market picking
				1.5f,
				fleetType,
				combat, // combatPts
				0f, // freighterPts
				0f, // tankerPts
				0f, // transportPts
				0f, // linerPts
				0f, // utilityPts
				0 // qualityBonus
		);


		LocationAPI location = FangspawnEntity.getContainingLocation();  // a star system, or hyperspace
		CampaignFleetAPI fleet = FleetFactoryV3.createFleet(params);
		location.addEntity(fleet);// adds our new fleet into the containing location
		fleet.setLocation(FangspawnEntity.getLocation().x, FangspawnEntity.getLocation().y);// positions fleet on top of the spawn entity

		if ((fleet == null) || fleet.isEmpty()) {
			return null;
		}

		// setting the below means: transponder off and more "go dark" use when traveling
		//fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);

		fleet.addAssignment(FleetAssignment.ORBIT_AGGRESSIVE, FangspawnEntity, 3f + (float) Math.random() * 2f);
		if ((float) Math.random() > 0.75f) {
			fleet.addAssignment(FleetAssignment.RAID_SYSTEM, FangspawnEntity, 250,
					"raiding around the " + FangspawnEntity.getName() + " star system");
		} else {
			fleet.addAssignment(FleetAssignment.RAID_SYSTEM, FangspawnEntity, 500,
					"raiding the " + FangspawnEntity.getName() + " star system");
		}
		fleet.addAssignment(FleetAssignment.GO_TO_LOCATION, FangspawnEntity, 20,
				"returning to " + FangspawnEntity.getName());
		fleet.addAssignment(FleetAssignment.ORBIT_PASSIVE, FangspawnEntity, 2f + 2f * (float) Math.random(),
				"orbiting " + FangspawnEntity.getName());

		fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_FLEET_TYPE, "fangFleet");
		fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PIRATE, true);
		fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_NO_MILITARY_RESPONSE, true);
		fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_FIGHT_TO_THE_LAST, true);

		return fleet;
	}

	protected SectorEntityToken Get_FursuitSupplier() {
		WeightedRandomPicker<SectorEntityToken> picker = new WeightedRandomPicker<SectorEntityToken>();
		for (StarSystemAPI system : Global.getSector().getStarSystems()) {
			float weight = 0f;
			if (system.getName().equals("Obsidian")) continue;
			float w = 11f;
			if (w > weight) weight = w;

			for (SectorEntityToken entity : system.getAllEntities()) {
				if (entity.getId().equals("anthracite")) {
					picker.add(entity, weight);
				}
			}
		}
		return picker.pick();
	}

}
