package data.hullmods;

import com.fs.starfarer.api.combat.BaseHullMod;
import com.fs.starfarer.api.combat.MutableShipStatsAPI;
import com.fs.starfarer.api.combat.ShipAPI;
import com.fs.starfarer.api.combat.ShipAPI.HullSize;
import com.fs.starfarer.api.impl.campaign.ids.Stats;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

public class draco extends BaseHullMod {

	private static final float SHIELD_DEBUFF = 0.40f;
    private static final float RANGE_DEBUFF = 0.75f;
    private static final float RANGE_CLAMP = 600f;
    private static final float VENT_BUFF = 1.75f;
    private static final float FIGHTER_MALUS = 0.50f;
    private static final float SPEED_BUFF = 0.50f;

    private static Map mag = new HashMap();
    static {
        mag.put(HullSize.FRIGATE, 20f);
        mag.put(HullSize.DESTROYER, 18f);
        mag.put(HullSize.CRUISER, 12f);
        mag.put(HullSize.CAPITAL_SHIP, 6f);
    }


	public void applyEffectsBeforeShipCreation(HullSize hullSize, MutableShipStatsAPI stats, String id) {
	    stats.getShieldDamageTakenMult().modifyMult(id, (1 + SHIELD_DEBUFF));

        stats.getWeaponRangeMultPastThreshold().modifyMult(id, (1 - RANGE_DEBUFF));
        stats.getWeaponRangeThreshold().modifyFlat(id, RANGE_CLAMP);

        stats.getMaxSpeed().modifyFlat(id, (Float) mag.get(hullSize));
        stats.getAcceleration().modifyMult(id, 1 + SPEED_BUFF);
        stats.getDeceleration().modifyMult(id, 1 + SPEED_BUFF);
        stats.getTurnAcceleration().modifyMult(id, 1 + SPEED_BUFF);
        stats.getVentRateMult().modifyMult(id, VENT_BUFF);

	}
    public void applyEffectsToFighterSpawnedByShip(ShipAPI fighter, ShipAPI ship, String id) {

        MutableShipStatsAPI stats = fighter.getMutableStats();

        stats.getMaxSpeed().modifyMult(id, 1f - FIGHTER_MALUS);
        stats.getArmorDamageTakenMult().modifyPercent(id, FIGHTER_MALUS * 100f);
        stats.getShieldDamageTakenMult().modifyPercent(id, FIGHTER_MALUS * 100f);
        stats.getHullDamageTakenMult().modifyPercent(id, FIGHTER_MALUS * 100f);

        stats.getMissileWeaponDamageMult().modifyPercent(id, FIGHTER_MALUS * 100f);
        stats.getBallisticWeaponDamageMult().modifyPercent(id, FIGHTER_MALUS * 100f);
        stats.getEnergyWeaponDamageMult().modifyPercent(id, FIGHTER_MALUS * 100f);

        fighter.setHeavyDHullOverlay();
    }


	public String getDescriptionParam(int index, HullSize hullSize) {
		if (index == 0) return "" + (int)Math.round((SHIELD_DEBUFF) * 100f) + "%";
        if (index == 1) return "" + (int)Math.round(RANGE_CLAMP) + " s.u.";
        if (index == 2) return "" + (int)Math.round((RANGE_DEBUFF) * 100f) + "%";
        if (index == 3) return "" + (int)Math.round((SPEED_BUFF) * 100f) + "%";
        if (index == 4) return "" + ((Float) mag.get(HullSize.FRIGATE)).intValue();
        if (index == 5) return "" + ((Float) mag.get(HullSize.DESTROYER)).intValue();
        if (index == 6) return "" + ((Float) mag.get(HullSize.CRUISER)).intValue();
        if (index == 7) return "" + ((Float) mag.get(HullSize.CAPITAL_SHIP)).intValue();
		if (index == 8) return "" + (int)Math.round((VENT_BUFF - 1) * 100f) + "%";
        if (index == 9) return "" + (int)Math.round((1 - FIGHTER_MALUS) * 100f) + "%";
		return null;
	}
}