package data.scripts;

import com.fs.starfarer.api.BaseModPlugin;
import com.fs.starfarer.api.Global;
import com.thoughtworks.xstream.XStream;
import data.campaign.dracomanager.Draco_RaiderBaseManager;
import data.campaign.dracomanager.Draco_RaiderNonEventFleetManager;
import data.campaign.fangmanager.Fang_RaiderBaseManager;
import data.campaign.fangmanager.Fang_RaiderNonEventFleetManager;
import data.campaign.fleets.DracoFleetManager;
import data.campaign.fleets.FangFleetManager;
import data.campaign.fleets.MessFleetManager;
import data.scripts.world.HMI_lootdracoget;
import data.scripts.world.HMI_lootfangget;
import data.scripts.world.HMI_lootmessget;
import exerelin.campaign.SectorManager;
import org.apache.log4j.Level;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;

public class HMI_modPlugin extends BaseModPlugin {

    public static boolean isExerelin = false;
    public static boolean Module_HMI = true;
    private static final String SETTINGS_FILE = "HMI_OPTIONS.ini";
    public static boolean Module_Brighton;
    public static boolean Module_Raider;

    private static void loadHMISettings() throws IOException, JSONException {
        JSONObject settings = Global.getSettings().loadJSON(SETTINGS_FILE);
        Module_Brighton = settings.getBoolean("brighton");
        Module_Raider = settings.getBoolean("hmiraiders");
    }

    public void configureXStream(XStream x) {
        x.alias("FangFleetManager", FangFleetManager.class);
        x.alias("DracoFleetManager", DracoFleetManager.class);
		x.alias("MessFleetManager", MessFleetManager.class);
        x.alias("HMI_lootmessget", HMI_lootmessget.class);
        x.alias("HMI_lootfangget", HMI_lootfangget.class);
        x.alias("HMI_lootdracoget", HMI_lootdracoget.class);
    }

    @Override
    public void onApplicationLoad() throws Exception {
        isExerelin = Global.getSettings().getModManager().isModEnabled("nexerelin");
        try {
            loadHMISettings();
        } catch (IOException | JSONException e) {
            Global.getLogger(HMI_modPlugin.class).log(Level.ERROR, "HMI_OPTIONS.ini loading failed!" + e.getMessage());
        }
    }

    public static void syncHMIScripts() {
        if (isExerelin && !SectorManager.getCorvusMode()) {
            return;
        }
        if (!Global.getSector().hasScript(HMI_lootmessget.class)) {
            Global.getSector().addScript(new HMI_lootmessget());
        }
        if (!Global.getSector().hasScript(HMI_lootfangget.class)) {
            Global.getSector().addScript(new HMI_lootfangget());
        }
        if (!Global.getSector().hasScript(HMI_lootdracoget.class)) {
            Global.getSector().addScript(new HMI_lootdracoget());
        }
    }

    public static void syncHMIScriptsExerelin() {
        if (isExerelin && !SectorManager.getCorvusMode()) {
            return;
        }
        if (!Global.getSector().hasScript(HMI_lootmessget.class)) {
            Global.getSector().addScript(new HMI_lootmessget());
        }
        if (!Global.getSector().hasScript(HMI_lootfangget.class)) {
            Global.getSector().addScript(new HMI_lootfangget());
        }
        if (!Global.getSector().hasScript(HMI_lootdracoget.class)) {
            Global.getSector().addScript(new HMI_lootdracoget());
        }
    }
    
    @Override
    public void onGameLoad(boolean newGame) {
        if (isExerelin) {
            syncHMIScriptsExerelin();
        } else {
            syncHMIScripts();
        }
    }

    @Override
    public void onNewGame() {
        if (isExerelin) {
            syncHMIScriptsExerelin();
        } else {
            syncHMIScripts();
        }
        HMI_modPlugin_alt.initHMI();

        if (Module_Brighton) {
            Global.getSector().getFaction("brighton").setShowInIntelTab(true);
            HMI_modPlugin_alt.initBrighton();
        } else {
            Global.getSector().getFaction("brighton").setShowInIntelTab(false);
        }

        if (Module_Raider) {
            Global.getSector().getFaction("fang").setShowInIntelTab(true);
            Global.getSector().getFaction("draco").setShowInIntelTab(true);
            HMI_modPlugin_alt.initHMIRaiders();
        } else {
            Global.getSector().getFaction("fang").setShowInIntelTab(false);
            Global.getSector().getFaction("draco").setShowInIntelTab(false);
        }
    }
}
	

