package data.scripts.world.systems;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.*;
import com.fs.starfarer.api.campaign.econ.MarketAPI;
import com.fs.starfarer.api.impl.campaign.ids.*;
import com.fs.starfarer.api.impl.campaign.procgen.DefenderDataOverride;
import com.fs.starfarer.api.impl.campaign.procgen.NebulaEditor;
import com.fs.starfarer.api.impl.campaign.procgen.themes.DerelictThemeGenerator;
import com.fs.starfarer.api.impl.campaign.terrain.HyperspaceTerrainPlugin;
import com.fs.starfarer.api.util.Misc;
import data.campaign.fleets.DracoFleetManager;
import data.campaign.fleets.FangFleetManager;
import data.campaign.fleets.MessFleetManager;

import java.awt.*;
import java.util.Random;

public class HMI_obsidian implements SectorGeneratorPlugin {

	private static final Random random = new Random();
    public static SectorEntityToken getSectorAccess() {
        return Global.getSector().getStarSystem("Obsidian").getEntityById("obsidian_star");
    }

	@Override
	public void generate(SectorAPI sector) {
		StarSystemAPI system = sector.createStarSystem("Obsidian");
		LocationAPI hyper = Global.getSector().getHyperspace();		
		system.setBackgroundTextureFilename("graphics/backgrounds/background2.jpg");

		PlanetAPI star = system.initStar("obsidian_star", // unique id for this star
										 StarTypes.RED_GIANT, // id in planets.json
										 800f,		// radius (in pixels at default zoom)
										 400,
							5f, // solar wind burn level
							0.2f, // flare probability
							4.0f); // CR loss multiplier, good values are in the range of 1-5
		
		system.setLightColor(new Color(225, 100, 100)); // light color in entire system, affects all entities
		system.getLocation().set(-27000, -27000);
		

        PlanetAPI obsidian1 = system.addPlanet("feldspar", star, "Feldspar", "lava", 245, 85, 1600, 200);

        Misc.initConditionMarket(obsidian1);
		obsidian1.getMarket().setSurveyLevel(MarketAPI.SurveyLevel.FULL);
		obsidian1.getMarket().addCondition(Conditions.VERY_HOT);
		obsidian1.getMarket().addCondition(Conditions.EXTREME_TECTONIC_ACTIVITY);
		obsidian1.getMarket().addCondition(Conditions.RARE_ORE_MODERATE);
		obsidian1.getMarket().addCondition(Conditions.ORE_MODERATE);
		obsidian1.getMarket().addCondition(Conditions.TOXIC_ATMOSPHERE);


        PlanetAPI obsidian2 = system.addPlanet("pyrite", star, "Pyrite", "rocky_metallic", 120, 125, 2000, 150);
		Misc.initConditionMarket(obsidian2);
	
        MarketAPI obsidian2PlanetMarket = Global.getFactory().createMarket("obsidian2_conditions", obsidian2.getName(), 2);
        obsidian2PlanetMarket.setPrimaryEntity(obsidian2);
        obsidian2PlanetMarket.setSurveyLevel(MarketAPI.SurveyLevel.FULL);
        obsidian2PlanetMarket.setFactionId("draco");
		obsidian2PlanetMarket.addCondition(Conditions.RUINS_SCATTERED);
		obsidian2PlanetMarket.addCondition(Conditions.TECTONIC_ACTIVITY);
		obsidian2PlanetMarket.addCondition(Conditions.ORE_RICH);
		obsidian2PlanetMarket.addCondition(Conditions.RARE_ORE_ULTRARICH);
		obsidian2PlanetMarket.addCondition(Conditions.HOT);
		obsidian2PlanetMarket.addCondition(Conditions.NO_ATMOSPHERE);

		obsidian2.setFaction("draco");
        obsidian2PlanetMarket.setPlanetConditionMarketOnly(true);
        obsidian2.setMarket(obsidian2PlanetMarket);

		int maxFleets = 5 + this.random.nextInt(2);
		DracoFleetManager DracoFleets = new DracoFleetManager(
				obsidian2, 1.0f, 2, maxFleets, 10.0f, 30, 50);
		system.addScript(DracoFleets);




		obsidian2.getSpec().setGlowTexture(Global.getSettings().getSpriteName("hab_glows", "asharu"));
		obsidian2.getSpec().setGlowColor(new Color(255,245,235,255));
		obsidian2.getSpec().setUseReverseLightForGlow(true);
		obsidian2.getSpec().setAtmosphereThicknessMin(25);
		obsidian2.getSpec().setAtmosphereThickness(0.2f);
		obsidian2.getSpec().setAtmosphereColor( new Color(80,90,100,120) );
		obsidian2.applySpecChanges();
        obsidian2.setCustomDescriptionId("hmi_pyrite");
		obsidian2.setInteractionImage("illustrations", "facility_explosion");		
		
        PlanetAPI obsidian3 = system.addPlanet("cinnibar", star, "Cinnibar", "lava", 80, 65, 2600, 365);

        Misc.initConditionMarket(obsidian3);
		obsidian3.getMarket().setSurveyLevel(MarketAPI.SurveyLevel.FULL);
		obsidian3.getMarket().addCondition(Conditions.HOT);
		obsidian3.getMarket().addCondition(Conditions.TECTONIC_ACTIVITY);
		obsidian3.getMarket().addCondition(Conditions.RARE_ORE_SPARSE);
		obsidian3.getMarket().addCondition(Conditions.ORE_ULTRARICH);
		obsidian3.getMarket().addCondition(Conditions.TOXIC_ATMOSPHERE);

        PlanetAPI obsidian4 = system.addPlanet("bauxite", star, "Bauxite", "barren-bombarded", 80, 95, 3500, 365);

        Misc.initConditionMarket(obsidian4);
		obsidian4.getMarket().setSurveyLevel(MarketAPI.SurveyLevel.FULL);
		obsidian4.getMarket().addCondition(Conditions.VERY_COLD);
		obsidian4.getMarket().addCondition(Conditions.NO_ATMOSPHERE);
		obsidian4.getMarket().addCondition(Conditions.ORE_RICH);
		obsidian4.getMarket().addCondition(Conditions.RARE_ORE_RICH);
		obsidian4.getMarket().addCondition(Conditions.HIGH_GRAVITY);


		system.addAsteroidBelt(star, 200, 3300, 512, 150, 325, Terrain.ASTEROID_BELT, "The Shards");
		system.addAsteroidBelt(star, 200, 3700, 512, 150, 400, Terrain.ASTEROID_BELT, "The Shards");
		system.addRingBand(star, "misc", "rings_dust0", 256f, 0, Color.white, 512f, 3200, 80f);
		system.addRingBand(star, "misc", "rings_dust0", 256f, 1, Color.white, 512f, 3400, 100f);
		system.addRingBand(star, "misc", "rings_dust0", 256f, 1, Color.white, 512f, 3600, 100f);

		PlanetAPI obsidian5 = system.addPlanet("bornite", star, "Bornite", "barren", 600, 200, 4250, 100);

		Misc.initConditionMarket(obsidian5);
		obsidian5.getMarket().setSurveyLevel(MarketAPI.SurveyLevel.FULL);
		obsidian5.getMarket().addCondition(Conditions.VERY_COLD);
		obsidian5.getMarket().addCondition(Conditions.THIN_ATMOSPHERE);
		obsidian5.getMarket().addCondition(Conditions.VOLATILES_TRACE);
		obsidian5.getMarket().addCondition(Conditions.ORE_SPARSE);
		obsidian5.getMarket().addCondition(Conditions.ORGANICS_TRACE);
		obsidian5.getMarket().addCondition(Conditions.RARE_ORE_SPARSE);
		obsidian5.getMarket().addCondition(Conditions.HIGH_GRAVITY);


		PlanetAPI obsidian6 = system.addPlanet("galena", star, "Galena", "hmi_crystalline", 80, 165, 6000, 400);
        obsidian6.setCustomDescriptionId("hmi_galena");
		obsidian6.setInteractionImage("illustrations", "crystalline");

		system.addAsteroidBelt(star, 100, 8300, 256, 150, 325, Terrain.ASTEROID_BELT, "The Shredders");
		system.addAsteroidBelt(star, 100, 8700, 256, 150, 400, Terrain.ASTEROID_BELT, "The Shredders");
		system.addRingBand(star, "misc", "rings_dust0", 256f, 0, Color.white, 256f, 8200, 80f);
		system.addRingBand(star, "misc", "rings_dust0", 256f, 1, Color.white, 512f, 8400, 100f);
		system.addRingBand(star, "misc", "rings_dust0", 256f, 1, Color.white, 256f, 8600, 100f);


		PlanetAPI obsidian7 = system.addPlanet("anthracite", star, "Anthracite", "barren-bombarded", 80, 125, 10000, 300);

		MarketAPI obsidian7PlanetMarket = Global.getFactory().createMarket("obsidian5_conditions", obsidian7.getName(), 2);
		obsidian7PlanetMarket.setPrimaryEntity(obsidian7);
		obsidian7PlanetMarket.setSurveyLevel(MarketAPI.SurveyLevel.FULL);
		obsidian7PlanetMarket.setFactionId("fang");
		obsidian7PlanetMarket.addCondition(Conditions.COLD);
		obsidian7PlanetMarket.addCondition(Conditions.METEOR_IMPACTS);
		obsidian7PlanetMarket.addCondition(Conditions.ORE_ULTRARICH);
		obsidian7PlanetMarket.addCondition(Conditions.RARE_ORE_SPARSE);
		obsidian7PlanetMarket.addCondition(Conditions.ORGANICS_PLENTIFUL);
		obsidian7PlanetMarket.addCondition(Conditions.THIN_ATMOSPHERE);
		obsidian7PlanetMarket.setPlanetConditionMarketOnly(true);
		obsidian7.setMarket(obsidian7PlanetMarket);

		obsidian7.setFaction("fang");
		obsidian7.setCustomDescriptionId("hmi_anthracite");
		obsidian7.setInteractionImage("illustrations", "mine");

		int maxFleets2 = 7 + this.random.nextInt(2);
		FangFleetManager FangFleets = new FangFleetManager(
				obsidian7, 1.0f, 3, maxFleets2, 10.0f, 20, 40);
		system.addScript(FangFleets);


		SectorEntityToken scrap1 = DerelictThemeGenerator.addSalvageEntity(system, Entities.EQUIPMENT_CACHE, Factions.DERELICT);
		scrap1.setId("fang_scrap1");
		scrap1.setCircularOrbit(star, 200, 8400, 200);
		Misc.setDefenderOverride(scrap1, new DefenderDataOverride(Factions.DERELICT, 0, 0, 0));
		scrap1.setDiscoverable(Boolean.TRUE);

		SectorEntityToken scrap2 = DerelictThemeGenerator.addSalvageEntity(system, Entities.SUPPLY_CACHE_SMALL, Factions.DERELICT);
		scrap2.setId("fang_scrap2");
		scrap2.setCircularOrbit(star, 340, 8200, 200);
		Misc.setDefenderOverride(scrap2, new DefenderDataOverride(Factions.DERELICT, 0, 0, 0));
		scrap2.setDiscoverable(Boolean.TRUE);

		SectorEntityToken scrap3 = DerelictThemeGenerator.addSalvageEntity(system, Entities.SUPPLY_CACHE, Factions.DERELICT);
		scrap3.setId("fang_scrap3");
		scrap3.setCircularOrbit(star, 90, 8600, 200);
		Misc.setDefenderOverride(scrap3, new DefenderDataOverride(Factions.DERELICT, 0, 0, 0));
		scrap3.setDiscoverable(Boolean.TRUE);

		JumpPointAPI jumpPoint = Global.getFactory().createJumpPoint("obsidian_jump_point1", "Obsidian Inner Jump-point");
		OrbitAPI orbit = Global.getFactory().createCircularOrbit(star, 0, 1400, 100);
		jumpPoint.setOrbit(orbit);
		orbit.setEntity(jumpPoint);
		jumpPoint.setRelatedPlanet(obsidian1);
		jumpPoint.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPoint);


		JumpPointAPI jumpPointOuter = Global.getFactory().createJumpPoint("obsidian_jump_point2", "Obsidian Outer Jump-point");
		OrbitAPI orbit_outer = Global.getFactory().createCircularOrbit(obsidian7, 0, 800, 60);
		jumpPointOuter.setOrbit(orbit_outer);
		orbit_outer.setEntity(jumpPointOuter);
		jumpPointOuter.setRelatedPlanet(obsidian7);
		jumpPointOuter.setStandardWormholeToHyperspaceVisual();
		system.addEntity(jumpPointOuter);

		system.autogenerateHyperspaceJumpPoints(true, true);
		orbit.setEntity(jumpPoint);
		cleanup(system);
	}

	void cleanup(StarSystemAPI system){
		HyperspaceTerrainPlugin plugin = (HyperspaceTerrainPlugin) Misc.getHyperspaceTerrain().getPlugin();
		NebulaEditor editor = new NebulaEditor(plugin);
		float minRadius = plugin.getTileSize() * 2f;

		float radius = system.getMaxRadiusInHyperspace();
		editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius * 0.5f, 0, 360f);
		editor.clearArc(system.getLocation().x, system.getLocation().y, 0, radius + minRadius, 0, 360f, 0.25f);
	}
}
